(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)

(* (cyclic) sequences *)

type +'a t

val empty: 'a t
val is_empty: 'a t -> bool

val cons: 'a -> 'a t -> 'a t
val snoc: 'a t -> 'a -> 'a t
val singleton: 'a -> 'a t
val append: 'a t -> 'a t -> 'a t

val head: 'a t -> 'a
val tail: 'a t -> 'a t

val fold_right: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
val fold_left: ('b -> 'a -> 'b) -> 'b -> 'a t -> 'b

(* forces all internal lazy values *)
(* val force: 'a t -> unit *)


(* [splittings f l] folds f through all decompositions [l=h@k] *)
val fold_splittings: ('a t -> 'a t -> 'b -> 'b) -> 'a t -> 'b -> 'b
(* special case for the boolean or *)
val splittings: ('a t -> 'a t -> bool) -> 'a t -> bool

(* [cyclic_positions f l] computes the "boolean or" of the [f x (k@h)] for all decompositions [l=h@x::k] *)
val cyclic_positions: ('a -> 'a t -> bool) -> 'a t -> bool


(* conversion between sequences and lists *)
val of_list: 'a list -> 'a t
val to_list: 'a t -> 'a list
val to_stream: 'a t -> 'a Stream.t

(* is a sequence reduced to a given singleton *)
val is_singleton: 'a -> 'a t -> bool

(* equality test *)
val equal: ('a -> 'a -> bool) -> 'a t -> 'a t -> bool
