(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)

open Formula


(* Tarjan-like equivalence classes *)
type typ = constr ref
and constr = R | L of typ

let rec repr t = 
  match !t with 
    | R -> t
    | L s -> let s = repr s in t := L s; s

let equal s t = 
  repr s == repr t

let unify s t = 
  let s = repr s in
  let t = repr t in
    if s != t then t := L s

let fresh (): typ = ref R


let square v (l: seq) = 
  let env = Array.make v None in
  let get_env i = match env.(i) with
    | None -> let st = fresh(),fresh() in env.(i) <- Some st; st
    | Some st -> st
  in
  let rec tgt s = function
    | `Var i -> let (si,ti) = get_env i in unify s si; ti 
    | `Rav i -> let (ti,si) = get_env i in unify s si; ti 
    | `One -> s
    | `Ten(a,b) -> tgt (tgt s a) b
    | `Seq l -> tgt_seq s l
    | `Wit(a,b) | `Opl(a,b) -> 
	let ta = tgt s a in 
	let tb = tgt s b in
	  unify ta tb; tb
  and tgt_seq s (l: seq) = Seq.fold_left tgt s (l:> t Seq.t) in
  let s = fresh() in
    equal s (tgt_seq s l)

